/* $Id: x86_compat.h,v 1.1.1.1 2000/05/21 14:40:03 mikpe Exp $
 * Performance-monitoring counters driver.
 * x86-specific compatibility definitions for 2.2.x/2.3.x kernels.
 *
 * Copyright (C) 2000  Mikael Pettersson
 */
#include <linux/config.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,40)
/* mmu_cr4_features is __initdata */

#define MAYBE_EXPORT_mmu_cr4_features	/*empty*/

static __inline__ void __write_cr4(unsigned int x)
{
	__asm__ __volatile__("movl %0,%%cr4" : : "r"(x));
}

static __inline__ unsigned int __read_cr4(void)
{
	unsigned int x;
	__asm__ __volatile__("movl %%cr4,%0" : "=r"(x));
	return x;
}

static __inline__ void __set_cr4_pce(void *ignore)
{
	__write_cr4(__read_cr4() | X86_CR4_PCE);
}

static __inline__ void __clear_cr4_pce(void *ignore)
{
	__write_cr4(__read_cr4() & ~X86_CR4_PCE);
}

static __inline__ void set_cr4_pce_global(void)
{
	__set_cr4_pce(NULL);
	smp_call_function(__set_cr4_pce, NULL, 1, 1);
}

static __inline__ void clear_cr4_pce_global(void)
{
	__clear_cr4_pce(NULL);
	smp_call_function(__clear_cr4_pce, NULL, 1, 1);
}

#define set_cr4_tsd_local()	__write_cr4(__read_cr4() | X86_CR4_TSD)

#else	/* >= 2.3.40, mmu_cr4_features is a global variable */

#define MAYBE_EXPORT_mmu_cr4_features	EXPORT_SYMBOL(mmu_cr4_features)

#ifdef CONFIG_SMP
static void update_cr4(void *ignore)
{
	__asm__ __volatile__("movl %0,%%cr4" : : "r"(mmu_cr4_features));
}
#endif

static __inline__ void set_cr4_pce_global(void)
{
	set_in_cr4(X86_CR4_PCE);
	smp_call_function(update_cr4, NULL, 1, 1);
}

static __inline__ void clear_cr4_pce_global(void)
{
	clear_in_cr4(X86_CR4_PCE);
	smp_call_function(update_cr4, NULL, 1, 1);
}

#define set_cr4_tsd_local()	set_in_cr4(X86_CR4_TSD)

#endif
